#ifndef INCLUDE_BON_DRIVER_WRAPPER_H
#define INCLUDE_BON_DRIVER_WRAPPER_H

class CBonDriver3Wrapper : public IBonDriver3
{
public:
    CBonDriver3Wrapper(IBonDriver3 *pBonWrapped, HMODULE hModuleWrapped, LPCTSTR pszName, IBonDriver **ppThis = NULL)
        : m_pBon(pBonWrapped), m_hModule(hModuleWrapped), m_tsAnalyzer(pszName), m_ppThis(ppThis) { if (m_ppThis) *m_ppThis = this; }
    ~CBonDriver3Wrapper() {}
#if 1 // IBonDriver
    const BOOL OpenTuner(void) { return m_pBon->OpenTuner(); }
    void CloseTuner(void) { return m_pBon->CloseTuner(); }
    const BOOL SetChannel(const BYTE bCh) { return static_cast<IBonDriver*>(m_pBon)->SetChannel(bCh); }
    const float GetSignalLevel(void) { return m_pBon->GetSignalLevel(); }
    const DWORD WaitTsStream(const DWORD dwTimeOut = 0) { return m_pBon->WaitTsStream(dwTimeOut); }
    const DWORD GetReadyCount(void) { return m_pBon->GetReadyCount(); }
    void PurgeTsStream(void) { m_pBon->PurgeTsStream(); }
    void Release(void) {
        if (m_ppThis) *m_ppThis = NULL;
        m_pBon->Release();
        ::FreeLibrary(m_hModule);
        delete this;
    }
    const BOOL GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(pDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(pDst, *pdwSize, *pdwRemain);
        return rv;
    }
    const BOOL GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(ppDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(*ppDst, *pdwSize, *pdwRemain);
        return rv;
    }
#endif

#if 1 // IBonDriver2
    LPCTSTR GetTunerName(void) { return m_pBon->GetTunerName(); }
    const BOOL IsTunerOpening(void) { return m_pBon->IsTunerOpening(); }
    LPCTSTR EnumTuningSpace(const DWORD dwSpace) { return m_pBon->EnumTuningSpace(dwSpace); }
    LPCTSTR EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) { return m_pBon->EnumChannelName(dwSpace, dwChannel); }
    const BOOL SetChannel(const DWORD dwSpace, const DWORD dwChannel) { return m_pBon->SetChannel(dwSpace, dwChannel); }
    const DWORD GetCurSpace(void) { return m_pBon->GetCurSpace(); }
    const DWORD GetCurChannel(void) { return m_pBon->GetCurChannel(); }
#endif

#if 1 // IBonDriver3
    const DWORD GetTotalDeviceNum(void) { return m_pBon->GetTotalDeviceNum(); }
    const DWORD GetActiveDeviceNum(void) { return m_pBon->GetActiveDeviceNum(); }
    const BOOL SetLnbPower(const BOOL bEnable) { return m_pBon->SetLnbPower(bEnable); }
#endif

private:
    IBonDriver3 *m_pBon;
    HMODULE m_hModule;
    CTsAnalyzer m_tsAnalyzer;
    IBonDriver **m_ppThis;
};

class CBonDriver2Wrapper : public IBonDriver2
{
public:
    CBonDriver2Wrapper(IBonDriver2 *pBonWrapped, HMODULE hModuleWrapped, LPCTSTR pszName, IBonDriver **ppThis = NULL)
        : m_pBon(pBonWrapped), m_hModule(hModuleWrapped), m_tsAnalyzer(pszName), m_ppThis(ppThis) { if (m_ppThis) *m_ppThis = this; }
    ~CBonDriver2Wrapper() {}
#if 1 // IBonDriver
    const BOOL OpenTuner(void) { return m_pBon->OpenTuner(); }
    void CloseTuner(void) { return m_pBon->CloseTuner(); }
    const BOOL SetChannel(const BYTE bCh) { return static_cast<IBonDriver*>(m_pBon)->SetChannel(bCh); }
    const float GetSignalLevel(void) { return m_pBon->GetSignalLevel(); }
    const DWORD WaitTsStream(const DWORD dwTimeOut = 0) { return m_pBon->WaitTsStream(dwTimeOut); }
    const DWORD GetReadyCount(void) { return m_pBon->GetReadyCount(); }
    void PurgeTsStream(void) { m_pBon->PurgeTsStream(); }
    void Release(void) {
        if (m_ppThis) *m_ppThis = NULL;
        m_pBon->Release();
        ::FreeLibrary(m_hModule);
        delete this;
    }
    const BOOL GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(pDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(pDst, *pdwSize, *pdwRemain);
        return rv;
    }
    const BOOL GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(ppDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(*ppDst, *pdwSize, *pdwRemain);
        return rv;
    }
#endif

#if 1 // IBonDriver2
    LPCTSTR GetTunerName(void) { return m_pBon->GetTunerName(); }
    const BOOL IsTunerOpening(void) { return m_pBon->IsTunerOpening(); }
    LPCTSTR EnumTuningSpace(const DWORD dwSpace) { return m_pBon->EnumTuningSpace(dwSpace); }
    LPCTSTR EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) { return m_pBon->EnumChannelName(dwSpace, dwChannel); }
    const BOOL SetChannel(const DWORD dwSpace, const DWORD dwChannel) { return m_pBon->SetChannel(dwSpace, dwChannel); }
    const DWORD GetCurSpace(void) { return m_pBon->GetCurSpace(); }
    const DWORD GetCurChannel(void) { return m_pBon->GetCurChannel(); }
#endif

private:
    IBonDriver2 *m_pBon;
    HMODULE m_hModule;
    CTsAnalyzer m_tsAnalyzer;
    IBonDriver **m_ppThis;
};

class CBonDriverWrapper : public IBonDriver
{
public:
    CBonDriverWrapper(IBonDriver *pBonWrapped, HMODULE hModuleWrapped, LPCTSTR pszName, IBonDriver **ppThis = NULL)
        : m_pBon(pBonWrapped), m_hModule(hModuleWrapped), m_tsAnalyzer(pszName), m_ppThis(ppThis) { if (m_ppThis) *m_ppThis = this; }
    ~CBonDriverWrapper() {}
#if 1 // IBonDriver
    const BOOL OpenTuner(void) { return m_pBon->OpenTuner(); }
    void CloseTuner(void) { return m_pBon->CloseTuner(); }
    const BOOL SetChannel(const BYTE bCh) { return static_cast<IBonDriver*>(m_pBon)->SetChannel(bCh); }
    const float GetSignalLevel(void) { return m_pBon->GetSignalLevel(); }
    const DWORD WaitTsStream(const DWORD dwTimeOut = 0) { return m_pBon->WaitTsStream(dwTimeOut); }
    const DWORD GetReadyCount(void) { return m_pBon->GetReadyCount(); }
    void PurgeTsStream(void) { m_pBon->PurgeTsStream(); }
    void Release(void) {
        if (m_ppThis) *m_ppThis = NULL;
        m_pBon->Release();
        ::FreeLibrary(m_hModule);
        delete this;
    }
    const BOOL GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(pDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(pDst, *pdwSize, *pdwRemain);
        return rv;
    }
    const BOOL GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain) {
        BOOL rv = m_pBon->GetTsStream(ppDst, pdwSize, pdwRemain);
        if (rv) m_tsAnalyzer.OnGetTsStream(*ppDst, *pdwSize, *pdwRemain);
        return rv;
    }
#endif

private:
    IBonDriver *m_pBon;
    HMODULE m_hModule;
    CTsAnalyzer m_tsAnalyzer;
    IBonDriver **m_ppThis;
};

#endif // INCLUDE_BON_DRIVER_WRAPPER_H
