#include <Windows.h>
#include "Util.h"
#include "TsAnalyzer.h"

#ifndef ASSERT
#include <cassert>
#define ASSERT assert
#endif

#define WM_QUERY_SETTEXT (WM_APP + 0)

CTsAnalyzer::CTsAnalyzer(LPCTSTR caption)
    : m_bufCount(0)
    , m_carryCount(0)
    , m_syncUnit(0)
    , m_fSynced(false)
    , m_tickTot(-1)
    , m_totwMsg(0)
    , m_hwnd(NULL)
    , m_hThread(NULL)
    , m_hThreadEvent(NULL)
    , m_fNeedToUpdate(false)
    , m_fInnerSetText(false)
{
    m_totwMsg = ::RegisterWindowMessage(TOTWRAP_BROADCAST);
    m_hThreadEvent = ::CreateEvent(NULL, FALSE, FALSE, NULL);
    ::lstrcpyn(m_caption, caption, _countof(m_caption));
}

CTsAnalyzer::~CTsAnalyzer()
{
    if (m_hwnd) {
        ::PostMessage(m_hwnd, WM_CLOSE, 0, 0);
        ::WaitForSingleObject(m_hThread, 5000);
    }
    if (m_hThread) {
        ::CloseHandle(m_hThread);
    }
    if (m_hThreadEvent) {
        ::CloseHandle(m_hThreadEvent);
    }
    DEBUG_OUT(TEXT("CTsAnalyzer::~CTsAnalyzer(): Done\n"));
}

void CTsAnalyzer::OnGetTsStream(const BYTE *pDst, DWORD dwSize, DWORD dwRemain)
{
    if (!m_fSynced) {
        // obt@𒙂߂
        ASSERT(m_bufCount < BUF_SIZE);
        int copyLen = min(dwSize, BUF_SIZE);
        if (copyLen > 0) {
            if (BUF_SIZE-copyLen > 0) ::memmove(m_buf, m_buf+copyLen, BUF_SIZE-copyLen);
            ::memcpy(m_buf+BUF_SIZE-copyLen, pDst+dwSize-copyLen, copyLen);
            m_bufCount += copyLen;
        }
        if (m_bufCount >= BUF_SIZE) {
            DEBUG_OUT(TEXT("CTsAnalyzer::OnGetTsStream(): Syncing\n"));

            // TSƓ݂
            m_syncUnit = select_unit_size(m_buf, m_buf + BUF_SIZE);
            if (188 <= m_syncUnit && m_syncUnit <= 320) {
                BYTE *pSync = resync(m_buf, m_buf + BUF_SIZE, m_syncUnit);
                if (pSync) {
                    // BŌ̃pPbgwb_Jz
                    BYTE *pSyncTail = pSync + (m_buf + BUF_SIZE - pSync)/m_syncUnit*m_syncUnit;
                    m_carryCount = static_cast<int>(m_buf + BUF_SIZE - pSyncTail);
                    ASSERT(m_carryCount < m_syncUnit);
                    if (m_carryCount > 0) ::memcpy(m_carry, m_buf+BUF_SIZE-m_carryCount, m_carryCount);
                    m_fSynced = true;
                }
            }
            m_bufCount = 0;
        }
    }
    else {
        // 1pPbgł邾̃f[^𒙂߂
        ASSERT(m_carryCount < m_syncUnit + 1);
        int copyLen = min((int)dwSize, m_syncUnit + 1 - m_carryCount);
        if (copyLen > 0) ::memcpy(m_carry + m_carryCount, pDst, copyLen);
        m_carryCount += copyLen;
        if (m_carryCount == m_syncUnit + 1) {
            // JzpPbg
            if (m_carry[0] != 0x47 || m_carry[m_syncUnit] != 0x47) {
                m_fSynced = false;
                return;
            }
            ProcessPacket(m_carry);

            // c̃pPbg
            ASSERT(copyLen >= 1);
            LPCBYTE curr = pDst + copyLen - 1;
            LPCBYTE tail = pDst + dwSize;
            for (; curr + m_syncUnit < tail; curr += m_syncUnit) {
                if (curr[0] != 0x47 || curr[m_syncUnit] != 0x47) {
                    m_fSynced = false;
                    return;
                }
                ProcessPacket(curr);
            }
            // ɌJz
            m_carryCount = static_cast<int>(tail - curr);
            if (m_carryCount > 0) ::memcpy(m_carry, curr, m_carryCount);
        }
    }
}

void CTsAnalyzer::ProcessPacket(const BYTE *pData)
{
    if ((((pData[1]&0x1f)<<8)|pData[2])/*pid*/ == 0x14 &&
        !(pData[1]&0x80)/*errorIndicator*/ &&
        (pData[1]&0x40)/*unitStart*/ &&
        ((pData[3]>>6)&0x03)/*scramblingControl*/ == 0)
    {
        int adaptControl = (pData[3]>>4)&0x03;
        if (adaptControl == 1 || adaptControl == 3) {
            LPCBYTE pPayload = pData + 4;
            if (adaptControl == 3) {
                // A_ve[VtB[hXLbv
                int adaptLength = pData[4];
                if (adaptLength > 182) return;
                pPayload += 1 + adaptLength;
            }
            int pointerField = pPayload[0];
            LPCBYTE pTable = pPayload + 1 + pointerField;
            if (pTable + 7 < pData + 188) {
                int tableID = pTable[0];
                if (tableID == 0x73/*TOT*/ || tableID == 0x70/*TDT*/) {
                    SYSTEMTIME st;
                    FILETIME ft;
                    if (AribToSystemTime(&pTable[3], &st) && ::SystemTimeToFileTime(&st, &ft)) {
                        CBlockLock lock(&m_totLock);
                        m_tickTot = ::GetTickCount();
                        m_stTot = st;
                        m_ftTot = ft;
                        OnTotUpdated();
                    }
                }
            }
        }
    }
}

void CTsAnalyzer::OnTotUpdated()
{
#ifdef DDEBUG_OUT
    TCHAR debug[256];
    ::wsprintf(debug, TEXT("CTsAnalyzer::OnTotUpdated(): %04d-%02d-%02dT%02d:%02d:%02d\n"),
               m_stTot.wYear, m_stTot.wMonth, m_stTot.wDay,
               m_stTot.wHour, m_stTot.wMinute, m_stTot.wSecond);
    DEBUG_OUT(debug);
#endif

    if (m_hThreadEvent && !m_hwnd) {
        m_hThread = ::CreateThread(NULL, 0, WndThread, this, 0, NULL);
        if (m_hThread) {
            // bZ[WL[ł(s)܂ő҂
            ::WaitForSingleObject(m_hThreadEvent, INFINITE);
        }
        if (!m_hwnd && m_hThread) {
            ::WaitForSingleObject(m_hThread, 5000);
            ::CloseHandle(m_hThread);
            m_hThread = NULL;
        }
    }
    m_fNeedToUpdate = true;
    if (m_hwnd) {
        // TOTEBhE^CgŒm点
        ::SendNotifyMessage(m_hwnd, WM_QUERY_SETTEXT, 0, 0);
    }
}

DWORD WINAPI CTsAnalyzer::WndThread(LPVOID pParam)
{
    CTsAnalyzer *pThis = static_cast<CTsAnalyzer*>(pParam);

    // EBhENXo^
    WNDCLASS wc = {0};
    wc.lpfnWndProc = WndProc;
    wc.hInstance = g_hModule;
    wc.lpszClassName = TOTWRAP_CLASSNAME;
    ATOM atom = ::RegisterClass(&wc);
    if (atom) {
        // EBhE𐶐ăbZ[W[vɓ
        pThis->m_hwnd = ::CreateWindow(MAKEINTATOM(atom), NULL, 0, 0, 0, 0, 0, HWND_MESSAGE, NULL, g_hModule, pThis);
        if (pThis->m_hwnd) {
            MSG msg;
            ::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE);
            ::SetEvent(pThis->m_hThreadEvent);
            while (::GetMessage(&msg, NULL, 0, 0) > 0) {
                ::DispatchMessage(&msg);
            }
        }
    }
    // n
    pThis->m_hwnd = NULL;
    if (atom) {
        ::UnregisterClass(MAKEINTATOM(atom), g_hModule);
    }
    ::SetEvent(pThis->m_hThreadEvent);
    return 0;
}

LRESULT CALLBACK CTsAnalyzer::WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    CTsAnalyzer *pThis = reinterpret_cast<CTsAnalyzer*>(::GetWindowLongPtr(hwnd, GWLP_USERDATA));
    switch (uMsg) {
    case WM_CREATE:
        {
            LPCREATESTRUCT pcs = reinterpret_cast<LPCREATESTRUCT>(lParam);
            ::SetWindowLongPtr(hwnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(pcs->lpCreateParams));
        }
        return 0;
    case WM_QUERY_SETTEXT:
        if (pThis->m_fNeedToUpdate) {
            TCHAR title[128 + TOTWRAP_CAPTION_MAX];
            {
                CBlockLock lock(&pThis->m_totLock);
                if (pThis->m_tickTot < 0) {
                    // TOT͌ݎ擾ĂȂ(insLen>=0)
                    ::wsprintf(title, TEXT("TOTWRAP N00 %s"), pThis->m_caption);
                }
                else {
                    // TOT͎擾Ă(insLen>=33)
                    ::wsprintf(title, TEXT("TOTWRAP Y33 %#08x %#08x %#08x %s"),
                               static_cast<DWORD>(pThis->m_tickTot),
                               pThis->m_ftTot.dwLowDateTime, pThis->m_ftTot.dwHighDateTime, pThis->m_caption);
                }
            }
            // EBhE^CgύX
            pThis->m_fInnerSetText = true;
            ::SendMessage(hwnd, WM_SETTEXT, 0, reinterpret_cast<WPARAM>(title));
            pThis->m_fInnerSetText = false;

            // XVu[hLXgbZ[WŒm点
            if (pThis->m_totwMsg) {
                CBlockLock lock(&pThis->m_totLock);
                ::PostMessage(HWND_BROADCAST, pThis->m_totwMsg, reinterpret_cast<WPARAM>(hwnd),
                              FileTimeToUnixTime(&pThis->m_ftTot));
            }
            pThis->m_fNeedToUpdate = false;
        }
        return 0;
    case WM_SETTEXT:
        // WM_QUERY_SETTEXToRłȂΕύX
        if (!pThis->m_fInnerSetText) return 0;
        break;
    case WM_DESTROY:
        ::PostQuitMessage(0);
        return 0;
    }
    return ::DefWindowProc(hwnd, uMsg, wParam, lParam);
}
